"use strict";

const _ = require('lodash');
const createClient = require('./src/createClient');
const formatResponse = require('./src/formatResponse');
const { promisify } = require('util');

async function callClientMethod(client, method, params) {
  method = method + 'Async';
  if(!_.isFunction(_.get(client, method))) {
    throw new Error('The specidied method ' + _.toString(method) + ' is not a supported function in this client');
  }

  return await client[method](params);
};

async function request({ url, method, headers }, params = {}, options = {}) {
  const client = await createClient(url, headers);
  const response = await callClientMethod(client, method, params);
  return formatResponse(response, options);
};

module.exports = {
  request
};
