"use strict";

const xml2js = require('xml2js');
const _ = require('lodash');
const chai = require('chai');
chai.use(require('chai-as-promised'));
const { expect } = chai;

const requestor = require('../index.js');

describe('XML Requests', () => {
  describe('requests', () => {
    it('should successively perform request', async () => {
      const config = {
        url: 'http://www.webservicex.com/globalweather.asmx?WSDL',
        method: 'GetCitiesByCountry'
      };

      const params = {
        CountryName: "Egypt"
      };

      const options = {
        explicitArray: false,
        tagNameProcessors: [ xml2js.processors.stripPrefix ]
      }

      const response = await requestor.request(config, params, options);
      const data = response.GetCitiesByCountryResult.NewDataSet.Table;
      expect(_.isArray(data)).to.be.true;
      _.forEach(data, (item) => {
        expect(_.isObject(item)).to.be.true;
      });
    });

    it('should test if method is not defined', async () => {
      const invalidMethod = 'invalidMethodNNN';
      const config = {
        url: 'http://www.webservicex.com/globalweather.asmx?WSDL',
        method: invalidMethod
      };

      const params = {
        CountryName: "Egypt"
      };

      const options = {
        explicitArray: false,
        tagNameProcessors: [ xml2js.processors.stripPrefix ]
      }

      const response = requestor.request(config, params, options);
      return expect(response).to.be.eventually.rejected.and.to.have.property('message').that.includes(invalidMethod);
    });

    it('should test if invalid options are sent', async () => {
      const config = {
        url: 'http://www.webservicex.com/globalweather.asmx?WSDL',
        method: 'GetCitiesByCountry'
      };

      const params = {
        CountryName: "Egypt"
      };

      const options = {
        explicitArray: false,
        invalidTagsNNN: true
      }

      const response = await requestor.request(config, params, options);
      const data = response.GetCitiesByCountryResult.NewDataSet.Table;

      expect(_.isArray(data)).to.be.true;
      _.forEach(data, (item) => {
        expect(_.isObject(item)).to.be.true;
      });
    });

    it('should test if invalid url is sent', async () => {
      const invalidurl = 'http://wwww.invalidurladwewae2e2wqeq23eqe3wqqde.com';
      const config = {
        url: invalidurl,
        method: 'GetCitiesByCountry'
      };

      const response = requestor.request(config);
      return expect(response).to.be.eventually.rejected.and.to.have.property('message').that.includes('invalidurladwewae2e2wqeq23eqe3wqqde.com');
    });
  });
});
